/*
 * drivers/pcmcia/ep93xx_pcmcia.h
 *
 * Copyright 2010 Atmark Techno, Inc. All Rights Reserved.
 *
 * Based on the Cirrus Logic ep93xx_pcmcia.h
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#ifndef __ASM_ARCH_PCMCIA_H__
#define __ASM_ARCH_PCMCIA_H__

#include <asm/arch/gpio.h>

/*
 * EEprom control signals.
 */
#define  GPIOA_EECLK                    (1<<0)
#define  GPIOA_EEDAT                    (1<<1)
#define  GPIOA_SLA0                     (1<<2)

/*
 * PCMCIA status signals = GPIO port F.
 */ 
#define EP93XX_PCMCIA_WP		(1<<0)
#define EP93XX_PCMCIA_MCD1		(1<<1)
#define EP93XX_PCMCIA_MCD2		(1<<2)
#define EP93XX_PCMCIA_MCBVD1		(1<<3)
#define EP93XX_PCMCIA_MCBVD2		(1<<4)
#define EP93XX_PCMCIA_VS1		(1<<5)
#define EP93XX_PCMCIA_READY		(1<<6)
#define EP93XX_PCMCIA_VS2		(1<<7)

#define EP93XX_GPIO_TO_IRQ(x)		((x) + 64)
#define EP93XX_PCMCIA_INT_WP		EP93XX_GPIO_TO_IRQ(EP93XX_GPIO_LINE_WP)		/* 80 */
#define EP93XX_PCMCIA_INT_MCD1		EP93XX_GPIO_TO_IRQ(EP93XX_GPIO_LINE_MCCD1)	/* 81 */
#define EP93XX_PCMCIA_INT_MCD2		EP93XX_GPIO_TO_IRQ(EP93XX_GPIO_LINE_MCCD2)	/* 82 */
#define EP93XX_PCMCIA_INT_MCBVD1	EP93XX_GPIO_TO_IRQ(EP93XX_GPIO_LINE_MCBVD1)	/* 83 */
#define EP93XX_PCMCIA_INT_MCBVD2	EP93XX_GPIO_TO_IRQ(EP93XX_GPIO_LINE_MCBVD2)	/* 84 */
#define EP93XX_PCMCIA_INT_VS1		EP93XX_GPIO_TO_IRQ(EP93XX_GPIO_LINE_VS1)	/* 85 */
#define EP93XX_PCMCIA_INT_READY		EP93XX_GPIO_TO_IRQ(EP93XX_GPIO_LINE_READY)	/* 86 */
#define EP93XX_PCMCIA_INT_VS2		EP93XX_GPIO_TO_IRQ(EP93XX_GPIO_LINE_VS2)	/* 87 */

/*
 * PCMCIA power = GPIO port A [3].
 */ 
#define EP93XX_PCMCIA_POWER		EP93XX_GPIO_LINE_EGPIO3


/*
 * Definitions to program the TPS2202A PCMCIA controller.
 */
#define AVCC_0V         			0x000
#define AVCC_33V        			0x004
#define AVCC_5V         			0x008
#define BVCC_0V         			0x000
#define BVCC_33V        			0x080
#define BVCC_5V         			0x040
#define AVPP_0V         			0x000
#define BVPP_0V         			0x000
#define ENABLE          			0x100
#define EE_ADDRESS      			(0x55<<9)

#define EP93XX_MAX_SOCK   (1)

#define EP93XX_PCMCIA_IO_ACCESS      (300)
#define EP93XX_PCMCIA_5V_MEM_ACCESS  (300)
#define EP93XX_PCMCIA_3V_MEM_ACCESS  (600)

#endif /* __ASM_ARCH_PCMCIA_H__ */
